/* eslint-disable unicorn/string-content */
import { describe, expect, it } from "vitest";
import { getClassName } from "../index";
import { getBuild } from "./utils";

describe("merges with existing className attribute", () => {
  const compileESBuild = getBuild("merge");

  it("string literal", async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      expectFiles: 2,
      javascript: /* tsx */ `
        export function Hello() {
          return (
            <div className="text-center" css="text-center">
              Hello, world!
            </div>
          );
        }
    `,
    });

    const clsName = getClassName("text-center");
    expect(files.js.text).toContain(`className: "text-center ${clsName}"`);
    expect(files.css.text).toMatch(`.${clsName} {\n  text-align: center;\n}`);
  });

  it("existing function", async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      expectFiles: 2,
      javascript: /* tsx */ `
        export function Hello() {
          return (
            <div className={({ isEntering }) => isEntering ? "enter" : "exit"} css="text-center">
              Hello, world!
            </div>
          );
        }
      `,
    });

    const clsName = getClassName("text-center");
    expect(files.js.text).toContain(
      `className: ({\n    isEntering\n  }) => _cx(isEntering ? "enter" : "exit", "${clsName}")`
    );
    expect(files.css.text).toMatch(`.${clsName} {\n  text-align: center;\n}`);
  });
});
