import tailwind from "tailwindcss";
import postcss from "postcss";
import type { ResolveTailwindOptions } from "./index";

export const { name: pkgName } = [require][0]("../package.json");
export const macroName = `${pkgName}/macro`;

interface LineColumn {
  line: number;
  column: number;
}

export interface SourceLocation {
  filename: string;
  start: LineColumn;
  end: LineColumn;
  text: string;
}

export interface StyleMapEntry {
  key: string;
  classNames: string[];
  location: SourceLocation;
}

export const tailwindDirectives = ["components", "utilities", "variants"] as const;

export type StyleMap = Map</* filename */ string, StyleMapEntry[]>;

export function createPostCSS({
  tailwindConfig,
  postCSSPlugins,
  prefix,
}: Pick<ResolveTailwindOptions, "tailwindConfig" | "postCSSPlugins" | "prefix">) {
  const post = postcss([
    tailwind({
      ...tailwindConfig,
      content: [{ raw: "<br>", extension: "html" }],
    }),
    ...postCSSPlugins,
  ]);

  return async (css: string) => {
    const result = await post.process((prefix ?? "") + css, { from: undefined });
    return result.css;
  };
}

export type Compile = ReturnType<typeof createPostCSS>;
